module StrMap = Map.Make (String)

let word_count file =
  let ic = open_in file in
  let rec loop acc =
    match In_channel.input_line ic with
    | None -> acc
    | Some s ->
        let l = String.split_on_char ' ' s in
        let acc =
          List.fold_left
            (fun acc s ->
              let c =
                try StrMap.find s acc with
                | Not_found -> 0
              in
              StrMap.add s (c + 1) acc)
            acc l
        in
        loop acc
  in
  let res = loop StrMap.empty in
  close_in ic;
  res

let jungle_count = word_count "jungle2.txt"
let () = StrMap.iter (fun s c -> Printf.printf "%s : %d\n" s c) jungle_count

module IntMap = Map.Make (Int)

let words_by_freq smap =
  StrMap.fold
    (fun w c imap ->
      let wlist =
        try IntMap.find c imap with
        | Not_found -> []
      in
      IntMap.add c (w :: wlist) imap)
    smap IntMap.empty

let () =
  IntMap.iter
    (fun c wlist -> Printf.printf "%d: %s\n" c (String.concat "," wlist))
    (words_by_freq jungle_count)
